@@ -56,6 +56,7 @@ module Control.Monad.Free.Church
   ( F(..)
   , improve
   , fromF
+  , iter
   , iterM
   , toF
   , retract
@@ -69,7 +70,7 @@ module Control.Monad.Free.Church
 import Control.Applicative
 import Control.Monad as Monad
 import Control.Monad.Fix
-import Control.Monad.Free hiding (retract, iterM, cutoff)
+import Control.Monad.Free hiding (retract, iter, iterM, cutoff)
 import qualified Control.Monad.Free as Free
 import Control.Monad.Reader.Class
 import Control.Monad.Writer.Class
@@ -87,8 +88,12 @@ import Prelude hiding (foldr)
 -- <http://comonad.com/reader/2011/free-monads-for-less-2/>
 newtype F f a = F { runF :: forall r. (a -> r) -> (f r -> r) -> r }
 
+-- | Tear down a 'Free' 'Monad' using iteration.
+iter :: (f a -> a) -> F f a -> a
+iter phi xs = runF xs id phi
+
 -- | Like iter for monadic values.
-iterM :: (Monad m, Functor f) => (f (m a) -> m a) -> F f a -> m a
+iterM :: Monad m => (f (m a) -> m a) -> F f a -> m a
 iterM phi xs = runF xs return phi
 
 instance Functor (F f) where
@@ -110,7 +115,7 @@ instance Bind (F f) where
   (>>-) = (>>=)
 
 instance Monad (F f) where
-  return a = F (\kp _ -> kp a)
+  return = pure
   F m >>= f = F (\kp kf -> m (\a -> runF (f a) kp kf) kf)
 
 instance MonadFix (F f) where
@@ -118,7 +123,7 @@ instance MonadFix (F f) where
     a = f (impure a)
     impure (F x) = x id (error "MonadFix (F f): wrap")
 
-instance (Foldable f, Functor f) => Foldable (F f) where
+instance Foldable f => Foldable (F f) where
     foldr f r xs = runF xs f (foldr (.) id) r
     {-# INLINE foldr #-}
 
@@ -169,7 +174,7 @@ retract (F m) = m return Monad.join
 hoistF :: (forall x. f x -> g x) -> F f a -> F g a
 hoistF t (F m) = F (\p f -> m p (f . t))
 
--- | The very definition of a free monoid is that given a natural transformation you get a monoid homomorphism.
+-- | The very definition of a free monad is that given a natural transformation you get a monad homomorphism.
 foldF :: Monad m => (forall x. f x -> m x) -> F f a -> m a
 foldF f (F m) = m return (Monad.join . f)
 
